//
// Simple wrapper to get rid of flickering
// screen with simulation visualisation
//
//  1.  call get_graphics with the dimensions of the drawing area
//  2.  draw figure using the return Graphics handle
//  3.  call 'refresh' to update the figure in the visible screen
//
//
//  Esa Hyyti, 2004
//

package elib;

import java.awt.*;
import java.applet.*;

public class DoubleBuffer
{
    private Image     offscreenI;
    private Graphics  offscreenG;
    private Rectangle offscreenR;
    private Component master;
    private Color     background;
    
    public DoubleBuffer( Component m )
    {
        master     = m;
        offscreenI = null;
        offscreenG = null;
        offscreenR = null;
        background = null;
    }

    public Graphics get_graphics( Rectangle r )
    {
        if ( r.width < 1 || r.height < 1 )
        {
            System.out.println( "DoubleBuffer: invalid width or height" );
            return null;
        }
        
        if ( offscreenI == null || !offscreenR.equals( r ) )
        {
            System.out.println( "Reserving memory for an offscreen image" );
            offscreenR = r;
            offscreenI = master.createImage( r.width, r.height );
            if ( offscreenI == null )
                return null;
            offscreenG = offscreenI.getGraphics( );
        }
        return offscreenG;
    }

    public Graphics get_graphics( )
    {
        return get_graphics( master.getBounds() );
    }

    public void clear_screen( Color c )
    {
        if ( offscreenI == null )
            return;
        offscreenG.setColor( c );
        offscreenG.fillRect( 0, 0, offscreenR.width, offscreenR.height );
    }

    public void clear_screen( )
    {
        clear_screen( background );
    }

    public void set_background( Color c )
    {
        background = c;
    }
    
    public void refresh( Graphics g )
    {
        if ( g != null && offscreenI != null )
            g.drawImage( offscreenI, 0, 0, master );
    }

    public void refresh( )
    {
        refresh( master.getGraphics() );
    }
}
